% This script simulates the linear wave equation with a sine wave as
% initial condition.
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% variables contained in params which are used more than once
Nxi        = params.geometry.Nxi    ;
nTimeSteps = params.time.nTimeSteps ;

% quantities related to the discretization
dxi     = params.geometry.L/Nxi                         ; % grid size
xi      = dxi*(1:Nxi)                                   ; % grid point vector
dt      = dxi*params.data.cfl/abs(params.data.velocity) ; % time step size
stencil = 0.5*[0,0,-1,0,1,0,0]                          ; % finite difference stencil

% adding variables to the params struct
params.data.D               = D_generalPeriodic(Nxi, dxi, stencil) ; % finite difference matrix
params.geometry.nDimensions = 1                                    ; % number of spatial dimensions
params.geometry.dxi         = dxi                                  ;
params.data.nVars           = 2                                    ; % number of physical variables
params.time.dt              = dt                                   ; 
params.time.t               = dt*(0:1:nTimeSteps)                  ; % discrete time vector

% preparation for the simulation
qs        = zeros(Nxi, params.data.nVars, nTimeSteps) ; % snapshot tensor
rho       = sin(xi')                                  ; % initial condition for density
u         = zeros(Nxi,1)                              ; % initial condition for velocity
qs(:,1,1) = rho                                       ;
qs(:,2,1) = u                                         ;

% time integration using an explicit 4th-order Runge-Kutta scheme
for n=2:nTimeSteps
    qs(:,:,n) = RK4(squeeze(qs(:,:,n-1)), dt, @(x)rhsString(x,params)) ;
end

% if onlyDensity is true, then velocity snapshots are removed from qs
onlyDensity = giveStructValue(params, 'data', 'onlyDensity', false) ;
if(onlyDensity)
    qs                = qs(:,1,:) ;
    params.data.nVars = 1         ;
end

clear Nxi nTimeSteps dxi xi dt stencil rho u onlyDensity