% This script produces snapshot data based on the analytic solution of the 
% linear wave equation with a Gaussian pulse as initial condition.
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% variables contained in params which are used more than once
Nxi        = params.geometry.Nxi    ;
L          = params.geometry.L      ; 
velocity   = params.data.velocity   ;
nTimeSteps = params.time.nTimeSteps ;

% quantities related to the sampling of the analytic solution
xi  = (1:Nxi)/(Nxi)*L                        ; % grid point vector
dxi = xi(2)-xi(1)                            ; % grid size
dt  = dxi*params.data.cfl/max(abs(velocity)) ; % time step size
t   = dt*(0:1:nTimeSteps)                    ; % discrete time vector

% adding variables to the params struct
params.time.t               = t           ;
params.geometry.nDimensions = length(Nxi) ; % number of spatial dimensions
params.time.dt              = dt          ; 
params.geometry.dxi         = dxi         ; 
params.data.nVars           = 2           ; % number of physical variables

% setting up the analytic solution
qs             = zeros(Nxi, params.data.nVars, nTimeSteps)                                                                                  ; % snapshot tensor
IC             = @(xi)params.data.amplitude*(exp(-(xi-L*params.data.relativeStartingPoint).^2/(params.data.relativeStandardDeviation*L)^2)) ; % initial condition
halfIC         = @(x)0.5*IC(x)                                                                                                              ;
shiftedRho0    = shiftFunction(halfIC,  velocity, 0, L)                                                                                     ;
negShiftedRho0 = shiftFunction(halfIC, -velocity, 0, L)                                                                                     ;
qs1            = analyticSnapshotMatrix(shiftedRho0, t(2:end), xi)                                                                          ; % right going component
qs2            = analyticSnapshotMatrix(negShiftedRho0, t(2:end), xi)                                                                       ; % left going component
qs(:,1,:)      = qs1+qs2                                                                                                                    ; % density snapshots
qs(:,2,:)      = velocity*(qs1-qs2)                                                                                                         ; % velcocity snapshots

% if onlyDensity is true, then velocity snapshots are removed from qs
onlyDensity = giveStructValue(params, 'data', 'onlyDensity', false) ;
if(onlyDensity)
    qs                = qs(:,1,:) ;
    params.data.nVars = 1         ;
end

clear Nxi L velocity nTimeSteps xi dxi dt t IC halfIC shiftedRho0 negShiftedRho0 qs1 qs2 onlyDensity