% This script creates the plots from the paper which are related to the 
% case "twoTransportedQuantitiesDensity".
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)

close all

% quantities needed for x and t axis in the plots
Nxi        = size(qs, 1)                     ; 
nTimeSteps = size(qs, 3)                     ;
xi         = params.geometry.dxi*(1:Nxi)     ; 
t          = params.time.dt*(0:1:nTimeSteps) ;
L          = params.geometry.L               ;
velocity   = params.data.velocity            ;

qsPlot = squeeze(qs(:,1,:))' ; % density snapshots

% plotting the density snapshots
figure
pcolor(xi/xi(end),t(2:end)/(L/velocity),qsPlot)
shading flat
xlabel('x/L')
ylabel('t/(L/c)')
c = caxis ;
colorbar
title('Density Snapshots')

% loading the results of the MS&R (before the cleaning iteration)
S                      = load('RESULTS/firstIter')                                       ;
firstIter              = S.sPodSystem                                                    ;
varNum                 = size(qs, 2)                                                     ;
negativeShiftsViaRot90 = giveStructValue(params, 'mode', 'negativeShiftsViaRot90', true) ;
if(negativeShiftsViaRot90)
    f                               = @(x)rot90(x,2)                                                                                                                                                    ;
    qsTildeFirstIterationRightGoing = shiftMatrix(params, reshape(firstIter.U{1}*firstIter.S{1}*firstIter.V{1}',Nxi,varNum,nTimeSteps), 1, cellfun(f,firstIter.shiftMatrices{1},'UniformOutput',false)) ; % right-going component of MS&R
    qsTildeFirstIterationLeftGoing  = shiftMatrix(params, reshape(firstIter.U{2}*firstIter.S{2}*firstIter.V{2}',Nxi,varNum,nTimeSteps), 1, cellfun(f,firstIter.shiftMatrices{2},'UniformOutput',false)) ; % left-going component of MS&R
else
    qsTildeFirstIterationRightGoing = shiftMatrix(params, reshape(firstIter.U{1}*firstIter.S{1}*firstIter.V{1}',Nxi,varNum,nTimeSteps), 1, firstIter.negShiftMatrices{1}) ; % right-going component of MS&R
    qsTildeFirstIterationLeftGoing  = shiftMatrix(params, reshape(firstIter.U{2}*firstIter.S{2}*firstIter.V{2}',Nxi,varNum,nTimeSteps), 1, firstIter.negShiftMatrices{2}) ; % left-going component of MS&R
end
qsTildeFirstIterationRightGoingPlot = squeeze(qsTildeFirstIterationRightGoing(:,1,:))' ;

% plotting the right-going component of MS&R
figure
pcolor(xi/xi(end),t(2:end)/(L/velocity),qsTildeFirstIterationRightGoingPlot)
shading flat
caxis(c)
xlabel('x/L')
ylabel('t/(L/c)')
colorbar
title('MS&R: Right Going Wave')

qsTildeFirstIterationLeftGoingPlot = squeeze(qsTildeFirstIterationLeftGoing(:,1,:))' ;

% plotting the left-going component of MS&R
figure
pcolor(xi/xi(end),t(2:end)/(L/velocity),qsTildeFirstIterationLeftGoingPlot)
shading flat
caxis(c)
xlabel('x/L')
ylabel('t/(L/c)')
colorbar
title('MS&R: Left Going Wave')

% approximation via MS&R (before cleaning iteration)
qsTildeFirstIteration     = qsTildeFirstIterationRightGoing+qsTildeFirstIterationLeftGoing ;
qsTildeFirstIterationPlot = squeeze(qsTildeFirstIteration(:,1,:))'                         ;

% plotting the error of the MS&R approximation
figure
pcolor(xi/xi(end),t(2:end)/(L/velocity),abs(qsTildeFirstIterationPlot-qsPlot))
shading flat
caxis(c)
xlabel('x/L')
ylabel('t/(L/c)')
colorbar
title('MS&R: Error')

% plotting the relative error over the cleaing iterations
figure
semilogy(relErrors)
xlabel('iteration')
ylabel('mean error')

qsTildePlot = squeeze(qsTilde(:,1,:))'; % sPOD approximation

% plotting the sPOD approximation
figure
pcolor(xi/xi(end),t(2:end)/(L/velocity),qsTildePlot)
shading flat
caxis(c)
xlabel('x/L')
ylabel('t/(L/c)')
colorbar
title('sPOD Approximation')

% plotting the error of the sPOD approximation
figure
pcolor(xi/xi(end),t(2:end)/(L/velocity),abs(qsPlot-qsTildePlot))
shading flat
xlabel('x/L')
ylabel('t/(L/c)')
colorbar
title('Error of the sPOD Approximation')

% computing the POD approximation
qsReshaped = reshape(qs, varNum*Nxi, size(qs,3)) ; % snapshot matrix
[U, S, V]  = svd(qsReshaped)                     ;
qsPOD      = U(:,1:2)*S(1:2,1:2)*V(:,1:2)'       ; % POD approximation with 2 modes

% plotting the POD approximation
figure
pcolor(xi/xi(end),t(2:end)/(L/velocity),qsPOD')
shading flat
caxis(c)
xlabel('x/L')
ylabel('t/(L/c)')
colorbar
title('POD Approximation')

% sampling of various constant shift velocities
sampledshiftVelocities = -1.3:0.01:1.3                             ;
nSamplingPoints        = length(sampledshiftVelocities)            ; % number of sampled shift velocities
S                      = NaN(nSamplingPoints, min(nTimeSteps,Nxi)) ; % initialization of the matrix storing the singular values for each shift velocity
for i=1:nSamplingPoints
    csi         = sampledshiftVelocities(i)                                ; % current shift velocity
    sPodSystemi = initShiftMatrix_constVelocity(params, csi)               ; % sPodSystemi is a struct containing the shift matrices
    qsShiftedi  = shiftMatrix(params, qs, 1, sPodSystemi.shiftMatrices{1}) ; % shifted snapshot matrix
    S(i,:)      = svd(reshape(qsShiftedi,Nxi,nTimeSteps))                  ;
end

% plotting the singular values for various shift velocities
figure
plot(sampledshiftVelocities,S,'ko','markers',1)
xlabel('shift velocity')
ylabel('singular values')
xlim([min(sampledshiftVelocities),max(sampledshiftVelocities)])