% This script loads the snapshot data from a file.
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

fprintf('loading snapshot data ...\n')

% derived data from params
nVars                = length(params.data.name)                                   ; % number of physical variables
scaleSnapshots       = giveStructValue(params, 'data', 'scaleSnapshots', false)   ;
meshCoarseningFactor = giveStructValue(params, 'data', 'meshCoarseningFactor', 1) ;

if(meshCoarseningFactor>1)
    disp(['mesh is coarsened with a factor of ' num2str(meshCoarseningFactor)])
end

% loading the first snapshot matrix and derive basic data from it
S          = load(params.data.name{1}) ;
qs1        = double(S.qs)              ; % first snapshot matrix
sizeOfQs1  = size(qs1)                 ;
nTimeSteps = sizeOfQs1(end)            ; % number of discrete time steps

% adding variables to the params struct
params.data.nVars           = nVars                                         ;
params.time.nTimeSteps      = nTimeSteps                                    ;
params.time.dt              = S.dt                                          ; % time step size
params.geometry.dxi         = S.dx*meshCoarseningFactor                     ; % grid size
params.geometry.nDimensions = length(params.geometry.dxi)                   ; % number of spatial dimensions
params.geometry.Nxi         = ceil(sizeOfQs1(1:end-1)/meshCoarseningFactor) ; % number of grid points for each dimension
if(isfield(S,'t'))
    params.time.t = S.t ; % discrete time vector
end

% vectors containing the mesh indices of the coarse mesh with respect to
% the original mesh
coarseMeshVectorX = 1:meshCoarseningFactor:sizeOfQs1(1) ;
if(params.geometry.nDimensions>1)
    coarseMeshVectorY = 1:meshCoarseningFactor:sizeOfQs1(2) ;
    if(isfield(S,'Y'))
        params.geometry.Y = S.Y(coarseMeshVectorY,coarseMeshVectorX) ; % matrix containing grid point vector along the y direction (e.g. as output from meshgrid)
    end
    if(isfield(S,'X'))
        params.geometry.X = S.X(coarseMeshVectorY,coarseMeshVectorX) ; % matrix containing grid point vector along the x direction (e.g. as output from meshgrid)
    end
end

% loading all snapshot matrices
switch params.geometry.nDimensions
    case 1 % 1D case
        qs1 = double(S.qs(coarseMeshVectorX,:))           ;
        qs  = NaN(params.geometry.Nxi, nVars, nTimeSteps) ; % snapshot tensor
        if(scaleSnapshots)
            qs(:,1,:) = qs1/max(max(abs(qs1))) ;
        else
            qs(:,1,:) = qs1 ;
        end
        for i=2:nVars
            S 	      = load(params.data.name{i})         ; 
            qs(:,i,:) = double(S.qs(coarseMeshVectorX,:)) ;
            if(scaleSnapshots)
            	qs(:,i,:) = qs(:,i,:)/max(max(abs(qs(:,i,:)))) ;
            end
        end
    case 2 % 2D case
        qs1         = double(S.qs(coarseMeshVectorX,coarseMeshVectorY,:))                    ;
        qs          = NaN(params.geometry.Nxi(1), params.geometry.Nxi(2), nVars, nTimeSteps) ; % snapshot tensor
        qs(:,:,1,:) = qs1                                                                    ;
        if(scaleSnapshots)
        	qs(:,:,1,:) = qs1/max(max(max(abs(qs1)))) ;
        else
            qs(:,:,1,:) = qs1 ;
        end
        for i=2:nVars
            S 	        = load(params.data.name{i})                           ; 
            qs(:,:,i,:) = double(S.qs(coarseMeshVectorX,coarseMeshVectorY,:)) ;
            if(scaleSnapshots)
            	qs(:,:,i,:) = qs(:,:,i,:)/max(max(max(abs(qs(:,:,i,:))))) ;
            end
        end
    otherwise
        error('The spatial dimension has to be between 1 and 2!')
end

disp('..done!')

clear nVars scaleSnapshots S qs1 sizeOfQs1 nTimeSteps