% This script creates the plots from the paper which are related to the 
% case "vortexPair".
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)

close all

qsPlot0 = squeeze(qs(:,:,1,1))' ; % snapshot at t=0
Y       = params.geometry.X'-1  ; % y-axis used for the plots
X       = params.geometry.Y'-1  ; % x-axis used for the plots
levels  = -270:20:270           ; % levels for the contour lines

% plotting the snapshot at t=0
figure
contour(X, Y, qsPlot0, levels, 'k')
xlabel('x')
ylabel('y')
title('Snapshot t=0')

% snapshot at t=0.075
t          = params.time.t               ;
t0075      = abs(t-0.075)                ; % auxiliary variable for finding the time discretization index closest to 0.075
idx0075    = find(min(t0075)==t0075)     ; % time discretization index closest to 0.075
qsPlot0075 = squeeze(qs(:,:,1,idx0075))' ; % snapshot at t=0.075

% plotting the snapshot at t=0.075
figure
contour(X, Y, qsPlot0075, levels, 'k')
xlabel('x')
ylabel('y')
title('Snapshot t=0.075')

% snapshot at t=0.27
t027      = abs(t-0.27)                ; % auxiliary variable for finding the time discretization index closest to 0.27
idx027    = find(min(t027)==t027)      ; % time discretization index closest to 0.27
qsPlot027 = squeeze(qs(:,:,1,idx027))' ; % snapshot at t=0.27

% plotting the snapshot at t=0.27
figure
contour(X, Y, qsPlot027, levels, 'k')
xlabel('x')
ylabel('y')
title('Snapshot t=0.27')

% sampling of various constant shift velocities
nTimeSteps              = params.time.nTimeSteps          ;
qsTest                  = qs(1:4:end,1:4:end,:,:)         ; % shift sampling is performed based on reduced snapshot matrix qTest
paramsTemp              = params                          ; % paramsTemp is like params but accounts for decreased Nxi and increased dxi
paramsTemp.geometry.Nxi = ceil(params.geometry.Nxi/4)     ;
paramsTemp.geometry.dxi = params.geometry.dxi*4           ;
sampledshiftVelocities  = linspace(-1.5,3.2,250)          ;
nSamplingPoints         = length(sampledshiftVelocities)  ; % number of sampled shift velocities
S                       = NaN(nSamplingPoints,nTimeSteps) ; % initialization of the matrix storing the singular values for each shift velocity
for i=1:nSamplingPoints
    csi         = sampledshiftVelocities(i)                                         ; % current shift velocity
    sPodSystemi = initShiftMatrix_constVelocity(paramsTemp, csi)                    ; % sPodSystemi is a struct containing the shift matrices
    qsShiftedi  = shiftMatrix(paramsTemp, qsTest, 1, sPodSystemi.shiftMatrices{1})  ; % shifted snapshot matrix
    S(i,:)      = svd(reshape(qsShiftedi,prod(paramsTemp.geometry.Nxi),nTimeSteps)) ;
end

% plotting the singular values for various shift velocities
figure
plot(sampledshiftVelocities, S, 'ko', 'markers', 1)
xlabel('shift velocity')
ylabel('singular values')
xlim([min(sampledshiftVelocities),max(sampledshiftVelocities)])

% computing the convergence criteria based on 10+14 modes
usedModes           = [10 14]                                                              ;
S                   = load('RESULTS/convergenceCriteria')                                  ;
orthoError          = S.orthoError                                                         ; % orthogonality errors
Sdiff               = S.Sdiff                                                              ; % singular values of residual
Sall                = S.Sall                                                               ; % singular values associated to the sPOD modes
sPodSystems         = S.sPodSystems                                                        ; % contains sPOD modes for each cleaning iteration
nCleaningIterations = params.mode.nCleaningIterations                                      ;
SratioPlot          = NaN(nCleaningIterations, 2)                                          ; % initializazion of the matrix containing the ratio of the biggest singular value of the shifted residual and the smallest singular value of the sPOD modes
orthoErrorPlot      = NaN(nCleaningIterations, 2)                                          ; % initialization of the matrix containing the maximum orthogonality error for each cleaning iteration and each velocity frame
relErrorsPlot       = NaN(nCleaningIterations, 1)                                          ; % initialization of the vector containing the relative mean error of each cleaning iteration
errorMeasure        = giveStructValue(params, 'mode', 'errorMeasure', @relErrorVectorNorm) ; % function to compute the relative mean error
for i=1:nCleaningIterations
    minSVIndex          = min(size(Sall{i}{1},1), usedModes(1))                          ; % number of considered modes for the secondary vortex pair
    SratioPlot(i,1)     = Sdiff{i}{1}(1)/Sall{i}{1}(minSVIndex,minSVIndex)               ;
    orthoErrorPlot(i,1) = max(abs(orthoError{i}{1}(1:minSVIndex)))                       ;
    minSVIndex          = min(size(Sall{i}{2},1), usedModes(2))                          ; % number of considered modes for the primary vortex pair
    SratioPlot(i,2)     = Sdiff{i}{2}(1)/Sall{i}{2}(minSVIndex,minSVIndex)               ;
    orthoErrorPlot(i,2) = max(abs(orthoError{i}{2}(1:minSVIndex)))                       ;
    qsTildeTemp         = additiveSubspaceCombination(params, sPodSystems{i}, usedModes) ; % sPOD approximation with 10+14 modes after i cleaning iterations
    relErrorsPlot(i)    = errorMeasure(qs, qs-qsTildeTemp)                               ;
end

% plotting the relative mean error over the cleaning iterations
figure
semilogy(relErrorsPlot)
xlabel('iteration')
ylabel('mean error')

% plotting the singular value ratio "SratioPlot" over the cleaning
% iterations for each vortex pair
figure
plot(SratioPlot(:,2))
hold on
plot(SratioPlot(:,1))
plot(ones(nCleaningIterations,1), 'k') ;
ylabel('SV dominance')
ylim([0.5 20])
legend('primary vortex pair','secondary vortex pair')

% plotting the maximum orthogonality error over the cleaning iterations for
% each vortex pair
figure
plot(orthoErrorPlot(:,2))
hold on
plot(orthoErrorPlot(:,1))
ylabel('ortho. error')
legend('primary vortex pair','secondary vortex pair')

% first sPOD mode of secondary vortex pair
Nxi      = params.geometry.Nxi                            ;
modePlot = reshape(sPodSystem.U{1}(:,1), Nxi(1), Nxi(2))' ;

% plotting the first sPOD mode of secondary vortex pair
figure
pcolor(X, Y, modePlot)
shading flat
xlabel('x')
ylabel('y')
title('1st Mode of Secondary Vortex Pair')

% second sPOD mode of secondary vortex pair
modePlot = reshape(sPodSystem.U{1}(:,2), Nxi(1), Nxi(2))'  ;

% plotting the second sPOD mode of secondary vortex pair
figure
pcolor(X, Y, modePlot)
shading flat
xlabel('x')
ylabel('y')
title('2nd Mode of Secondary Vortex Pair')

% third sPOD mode of secondary vortex pair
modePlot = reshape(sPodSystem.U{1}(:,3), Nxi(1), Nxi(2))'  ;

% plotting the third sPOD mode of secondary vortex pair
figure
pcolor(X, Y, modePlot)
shading flat
xlabel('x')
ylabel('y')
title('3rd Mode of Secondary Vortex Pair')

% first sPOD mode of primary vortex pair
modePlot = reshape(sPodSystem.U{2}(:,1), Nxi(1), Nxi(2))'  ;

% plotting the first sPOD mode of primary vortex pair
figure
pcolor(X, Y, modePlot)
shading flat
xlabel('x')
ylabel('y')
title('1st Mode of Primary Vortex Pair')

% second sPOD mode of primary vortex pair
modePlot = reshape(sPodSystem.U{2}(:,2), Nxi(1), Nxi(2))'  ;

% plotting the second sPOD mode of primary vortex pair
figure
pcolor(X, Y, modePlot)
shading flat
xlabel('x')
ylabel('y')
title('2nd Mode of Primary Vortex Pair')

% third sPOD mode of primary vortex pair
modePlot = reshape(sPodSystem.U{2}(:,3), Nxi(1), Nxi(2))'  ;

% plotting the third sPOD mode of primary vortex pair
figure
pcolor(X, Y, modePlot)
shading flat
xlabel('x')
ylabel('y')
title('3rd Mode of Primary Vortex Pair')

% amplitudes of the first three modes of the secondary vortex pair
amplitudePlot1 = sPodSystem.V{1}(:,1)*sPodSystem.S{1}(1,1) ;
amplitudePlot2 = sPodSystem.V{1}(:,2)*sPodSystem.S{1}(2,2) ;
amplitudePlot3 = sPodSystem.V{1}(:,3)*sPodSystem.S{1}(3,3) ;

% plotting the amplitudes of the first three modes of the secondary vortex 
% pair
figure
plot(t, amplitudePlot1)
hold on
plot(t, amplitudePlot2)
plot(t, amplitudePlot3)
xlabel('time')
title('Secondary Vortex Pair')
legend('1st amplitude', '2nd amplitude', '3rd amplitude')

% amplitudes of the first three modes of the primary vortex pair
amplitudePlot1 = sPodSystem.V{2}(:,1)*sPodSystem.S{2}(1,1) ;
amplitudePlot2 = sPodSystem.V{2}(:,2)*sPodSystem.S{2}(2,2) ;
amplitudePlot3 = sPodSystem.V{2}(:,3)*sPodSystem.S{2}(3,3) ;

% plotting the amplitudes of the first three modes of the primary vortex 
% pair
figure
plot(t, amplitudePlot1)
hold on
plot(t, amplitudePlot2)
plot(t, amplitudePlot3)
xlabel('time')
title('Primary Vortex Pair')
legend('1st amplitude', '2nd amplitude', '3rd amplitude')

% computing the errors of POD and sPOD approximations for different mode
% numbers
rMax             = 30                                                                         ; % maximum number of modes
[U, S, V]        = svds(reshape(qs,prod(Nxi),nTimeSteps), rMax)                               ; % SVD of snapshot matrix
errPOD           = NaN(rMax, 1)                                                               ; % initialization of the vector containing the errors of the POD approximations
errsPOD          = NaN(rMax, 1)                                                               ; % initialization of the vector containing the errors of the sPOD approximations
[~, indexS]      = sort([diag(full(sPodSystem.S{1}));diag(full(sPodSystem.S{2}))], 'descend') ; % sorting the singular values associated to the sPOD modes
nModesFirstFrame = size(sPodSystem.S{1}, 1)                                                   ; % number of computed sPOD modes for the secondary vortex pair
for i=1:rMax
	errPOD(i)   = errorMeasure(qs, qs-reshape(U(:,1:i)*S(1:i,1:i)*V(:,1:i)',Nxi(1),Nxi(2),1,nTimeSteps)) ;
	nMode1      = sum(indexS(1:i)<=nModesFirstFrame)                                                     ; % current number of sPOD modes for the secondary vortex pair
	nMode2      = sum(indexS(1:i)>nModesFirstFrame)                                                      ; % current number of sPOD modes for the primary vortex pair
	qsTildeTemp = additiveSubspaceCombination(params, sPodSystem, [nMode1,nMode2])                       ; % current sPOD approximation
	errsPOD(i)  = errorMeasure(qs, qs-qsTildeTemp)                                                       ;
end

% plotting the POD and sPOD errors over the number of modes
figure
semilogy(errPOD)
hold on
semilogy(errsPOD)
xlabel('number of modes')
ylabel('relative error')
legend('POD','sPOD')

% plotting the snapshots at t=0.27 (pcolor plot)
figure
pcolor(X, Y, qsPlot027)
shading flat
xlabel('x')
ylabel('y')
title('Snapshot t=0.27')

% computing the sPOD approximation with 6+9 modes
nmode1         = 6                                                                ; % number of modes for the secondary vortex pair
nmode2         = 9                                                                ; % number of modes for the primary vortex pair
qsTildeTemp    = additiveSubspaceCombination(params, sPodSystem, [nMode1,nMode2]) ; % sPOD approximation with 6+9 modes
qsTildePlot027 = squeeze(qsTildeTemp(:,:,1,idx027))'                              ; % sPOD approximation at t=0.27

% plotting the sPOD approximation at t=0.27 (pcolor plot)
figure
pcolor(X, Y, qsTildePlot027)
shading flat
xlabel('x')
ylabel('y')
title('sPOD approximation at t=0.27')

% plotting the sPOD approximation at t=0.27
figure
contour(X, Y, qsTildePlot027, levels, 'k')
xlabel('x')
ylabel('y')
title('sPOD approximation at t=0.27')

% computing the sPOD approximation with 15 modes
qsTildePOD        = reshape(U(:,1:15)*S(1:15,1:15)*V(:,1:15)', Nxi(1), Nxi(2), 1, nTimeSteps) ; % POD approximation with 15 modes
qsTildePODPlot027 = squeeze(qsTildePOD(:,:,1,idx027))'                                        ; % POD approximation at t=0.27

% plotting the POD approximation at t=0.27 (pcolor plot)
figure
pcolor(X, Y, qsTildePODPlot027)
shading flat
xlabel('x')
ylabel('y')
title('POD approximation at t=0.27')

% plotting the POD approximation at t=0.27
figure
contour(X, Y, qsTildePODPlot027, levels, 'k')
xlabel('x')
ylabel('y')
title('POD approximation at t=0.27')